module Main exposing (..)

import Api exposing (fetchLessons)
import Browser
import Browser.Navigation as Nav
import Dict exposing (Dict)
import Element exposing (..)
import Element.Background as Background
import Element.Border as Border
import Element.Events exposing (onClick)
import Element.Font as Font
import Router exposing (parseUrl, router)
import Types exposing (Lessons, Model, Msg(..), Route(..), ServerResponse(..))
import Url



-- state


sampleLessons : Lessons
sampleLessons =
    Dict.fromList []


initialState : Url.Url -> Nav.Key -> Model
initialState url key =
    let
        route =
            parseUrl url
    in
    { isLoading = False
    , lessons = sampleLessons
    , route = route
    , key = key
    }



-- update


update : Msg -> Model -> ( Model, Cmd Msg )
update msg model =
    case msg of
        FetchLessons (Ok serres) ->
            -- let
            --     _ =
            --         Debug.log "hi" serres
            -- in
            case serres of
                OkResponse lessons ->
                    ( { model | lessons = lessons, isLoading = False }, Cmd.none )

                ErrorResponse _ ->
                    ( { model | isLoading = False }, Cmd.none )

        FetchLessons (Err _) ->
            ( { model | isLoading = False }, Cmd.none )

        FetchLesson res ->
            ( { model | isLoading = False }, Cmd.none )

        LinkClicked urlRequest ->
            let
                _ =
                    Debug.log "url request" urlRequest
            in
            case urlRequest of
                Browser.Internal url ->
                    ( model, Nav.pushUrl model.key (Url.toString url) )

                Browser.External href ->
                    ( model, Nav.load href )

        UrlChanged url ->
            let
                newRoute =
                    parseUrl url
            in
            ( { model | route = newRoute }, Cmd.none )



-- SUBSCRIPTIONS


subscriptions : Model -> Sub Msg
subscriptions _ =
    Sub.none


view : Model -> Browser.Document Msg
view model =
    let
        ( _, _, html ) =
            router model

        --     _ =
        --         Debug.log "model" model
    in
    { title = "Prosody"
    , body = [ html ]
    }


init : flags -> Url.Url -> Nav.Key -> ( Model, Cmd Msg )
init flags url key =
    Debug.log "Init flags" flags
        -- |> (\_ -> Debug.log "url" url)
        -- |> (\_ -> Debug.log "key" key)
        -- |> (\_ -> Debug.log "Initial State" initialState)
        |> (\_ -> ( initialState url key, fetchLessons ))


main : Program () Model Msg
main =
    Browser.application
        { init = init
        , view = view
        , update = update
        , subscriptions = subscriptions
        , onUrlRequest = LinkClicked
        , onUrlChange = UrlChanged
        }


color =
    { blue = rgb255 0x72 0x9F 0xCF
    , darkCharcoal = rgb255 0x2E 0x34 0x36
    , lightBlue = rgb255 0xC5 0xE8 0xF7
    , lightGrey = rgb255 0xE0 0xE0 0xE0
    , white = rgb255 0xFF 0xFF 0xFF
    }
