module Router exposing (parseUrl, router)

import Api exposing (fetchLesson)
import Dict
import Element exposing (..)
import Homepage
import Html exposing (Html)
import Lessonpage
import Types exposing (Model, Msg(..), Route(..))
import Url
import Url.Parser as Parser exposing ((</>), Parser)


routeParser : Parser (Route -> a) a
routeParser =
    Parser.oneOf
        [ Parser.map Root Parser.top
        , Parser.map LessonR (Parser.s "lesson" </> Parser.int </> Parser.int)
        ]


parseUrl : Url.Url -> Route
parseUrl url =
    Parser.parse routeParser url |> Maybe.withDefault NotFound


router : Model -> ( Model, Cmd Msg, Html Msg )
router model =
    case model.route of
        Root ->
            ( model, Cmd.none, Homepage.page model )

        LessonR lid cid ->
            lessonLoader lid cid model

        NotFound ->
            ( model, Cmd.none, notFound )


lessonLoader : Int -> Int -> Model -> ( Model, Cmd Msg, Html Msg )
lessonLoader lid cid model =
    case Dict.get lid model.lessons of
        Just lesson ->
            ( model, Cmd.none, Lessonpage.page lesson cid )

        Nothing ->
            ( { model | isLoading = True }, fetchLesson lid, loadingPage )


notFound : Html Msg
notFound =
    layout [] <|
        el
            [ centerX ]
            (text "404")


loadingPage : Html Msg
loadingPage =
    layout [] <|
        el [ centerX, padding 20 ] (text "Loading lesson...")
